/**-----------------------------------------------------------------------------------------
* Copyright © 2025 Progress Software Corporation. All rights reserved.
* Licensed under commercial license. See LICENSE.md in the project root for more information
*-------------------------------------------------------------------------------------------*/
import { Injectable } from '@angular/core';
import { BehaviorSubject, Subject } from 'rxjs';
import * as i0 from "@angular/core";
const emptyDateRange = () => ({
    start: new Date(0),
    end: new Date(0),
    text: '',
    shortText: ''
});
/**
 * A service for publishing the view state and actions to the Scheduler.
 */
export class ViewStateService {
    /**
     * A stream for publishing the visible date range of the current view to the Scheduler.
     */
    dateRange;
    /**
     * A stream for publishing the changes to the selected date that are initiated from the view.
     */
    nextDate;
    /**
     * A stream fro navigating from the current to another view.
     */
    navigate;
    /**
     * A stream for navigating from the current to another view.
     */
    viewEvent;
    /**
     * A stream for indicating that the view layout finished.
     */
    layoutEnd;
    /**
     * A stream for indicating that the view options has changed.
     */
    optionsChange;
    /**
     * A stream indicating that the user has started making a new slot selection.
     */
    slotSelectionStart;
    /**
     * A stream indicating that the user has dragged over a different slot while making a selection.
     */
    slotSelectionDrag;
    /**
     * A stream indicating that the user has finished making a slot selection.
     */
    slotSelectionEnd;
    /**
     * @hidden
     */
    toggleWorkHours = new Subject();
    /**
     * @hidden
     */
    toolbarVisibilityByView = new Map();
    dateRangeSource = new BehaviorSubject(emptyDateRange());
    nextDateSource = new Subject();
    navigateSource = new Subject();
    viewEventSource = new Subject();
    layoutEndSource = new Subject();
    optionsChangeSource = new Subject();
    slotSelectionStartSource = new Subject();
    slotSelectionStartDragSource = new Subject();
    slotSelectionStartEndSource = new Subject();
    constructor() {
        this.dateRange = this.dateRangeSource.asObservable();
        this.nextDate = this.nextDateSource.asObservable();
        this.navigate = this.navigateSource.asObservable();
        this.viewEvent = this.viewEventSource.asObservable();
        this.layoutEnd = this.layoutEndSource.asObservable();
        this.optionsChange = this.optionsChangeSource.asObservable();
        this.slotSelectionStart = this.slotSelectionStartSource.asObservable();
        this.slotSelectionDrag = this.slotSelectionStartDragSource.asObservable();
        this.slotSelectionEnd = this.slotSelectionStartEndSource.asObservable();
    }
    /**
     * Publishes the date that will be displayed by the Scheduler
     * typically as a result from processing a navigation action.
     */
    notifyNextDate(date) {
        this.nextDateSource.next(date);
    }
    /**
     * Publishes the visible date range of the view.
     * The view will calculate and set the new data range when
     * the selected date changes.
     */
    notifyDateRange(range) {
        this.dateRangeSource.next(range);
    }
    /**
     * Notifies the Scheduler that the view has completed its layout.
     */
    notifyLayoutEnd() {
        this.layoutEndSource.next();
    }
    /**
     * Navigates to another view.
     */
    navigateTo(args) {
        this.navigateSource.next(args);
    }
    /**
     * Notifies the Scheduler that the view options have been changed.
     */
    notifyOptionsChange(changes) {
        this.optionsChangeSource.next(changes);
    }
    /**
     * Emits a DOM event to the Scheduler.
     */
    emitEvent(name, args) {
        this.viewEventSource.next({
            name: name,
            args: args
        });
    }
    /** @hidden */
    notifySlotSelectionStart(selection) {
        this.slotSelectionStartSource.next(selection);
    }
    /** @hidden */
    notifySlotSelectionDrag(selection) {
        this.slotSelectionStartDragSource.next(selection);
    }
    /** @hidden */
    notifySlotSelectionEnd(selection) {
        this.slotSelectionStartEndSource.next(selection);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: ViewStateService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: ViewStateService, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: ViewStateService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return []; } });
